﻿using System.Linq;
using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis.Symbolic;
using HeuristicLab.Problems.DataAnalysis.Symbolic.Regression;

namespace HeuristicLab.OSGAEvaluator {
  [Item("OSGAPredictionCountsAnalyzer", "An analyzer which records the counts of the rejected offspring predicted by the evaluator without evaluating all rows.")]
  [StorableClass]

  public class OSGAPredictionCountsAnalyzer : SymbolicDataAnalysisSingleObjectiveAnalyzer {
    private const string EvaluatorParameterName = "Evaluator";
    private const string ResultCollectionParameterName = "Results";

    public ILookupParameter<SymbolicRegressionSingleObjectiveOsgaEvaluator> EvaluatorParameter {
      get { return (ILookupParameter<SymbolicRegressionSingleObjectiveOsgaEvaluator>)Parameters[EvaluatorParameterName]; }
    }

    public OSGAPredictionCountsAnalyzer() {
      Parameters.Add(new LookupParameter<SymbolicRegressionSingleObjectiveOsgaEvaluator>(EvaluatorParameterName));
    }

    protected OSGAPredictionCountsAnalyzer(OSGAPredictionCountsAnalyzer original, Cloner cloner) : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new OSGAPredictionCountsAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      var evaluator = EvaluatorParameter.ActualValue;
      if (evaluator == null)
        return base.Apply();

      var rejectedStats = evaluator.RejectedStats;
      var totalStats = evaluator.TotalStats;

      if (rejectedStats.Rows == 0 || totalStats.Rows == 0)
        return base.Apply();

      ResultCollection predictionResults;
      DataTable rejectedStatsTable, totalStatsTable;
      if (!ResultCollection.ContainsKey("OS Prediction")) {
        predictionResults = new ResultCollection();
        rejectedStatsTable = new DataTable("Rejected Stats");
        foreach (var rowName in rejectedStats.RowNames) {
          rejectedStatsTable.Rows.Add(new DataRow(rowName) { VisualProperties = { StartIndexZero = true, ChartType = DataRowVisualProperties.DataRowChartType.Columns, LineWidth = 1 } });
        }
        predictionResults.Add(new Result("Rejected Stats", rejectedStatsTable));
        totalStatsTable = new DataTable("Total Stats");
        foreach (var rowName in totalStats.RowNames) {
          totalStatsTable.Rows.Add(new DataRow(rowName) { VisualProperties = { StartIndexZero = true, ChartType = DataRowVisualProperties.DataRowChartType.Columns, LineWidth = 1 } });
        }
        predictionResults.Add(new Result("Total Stats", totalStatsTable));

        ResultCollection.Add(new Result("OS Prediction", predictionResults));
      } else {
        predictionResults = (ResultCollection)ResultCollection["OS Prediction"].Value;
        rejectedStatsTable = (DataTable)predictionResults["Rejected Stats"].Value;
        totalStatsTable = (DataTable)predictionResults["Total Stats"].Value;
      }

      int i = 0;
      foreach (var rowName in rejectedStats.RowNames) {
        // add a padding zero below to prevent columns from being cut off to the left
        rejectedStatsTable.Rows[rowName].Values.Replace(new[] { 0d }.Concat(Enumerable.Range(0, rejectedStats.Columns).Select(j => (double)rejectedStats[i, j])).Concat(new[] { 0d }));
        ++i;
      }
      i = 0;
      foreach (var rowName in totalStats.RowNames) {
        totalStatsTable.Rows[rowName].Values.Replace(new[] { 0d }.Concat(Enumerable.Range(0, totalStats.Columns).Select(j => (double)totalStats[i, j])).Concat(new[] { 0d }));
        ++i;
      }
      return base.Apply();
    }
  }
}
